#!/usr/local/bin/python
# -*- coding: iso-8859-1 -*-

## Copyright 2005 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005 by Walter Drwald
##
## All Rights Reserved
##
## See orasql.py for the license


import sys, difflib

from ll import orasql, ansistyle


color4comment = 0100
color4addedfile = 0002
color4addedline = 0020
color4removedfile = 0001
color4removedline = 0010
color4changedfile = 0004
color4changedline = 0040
color4pos = 0100


def cs(cursor, color):
	connectstring = cursor.connection.connectstring()
	if color:
		connectstring = ansistyle.Text(0040, connectstring)
	return connectstring


def comment(msg, color):
	msg = "-- %s" % msg
	if color:
		msg = ansistyle.Text(0100, msg)
	return msg


def gettimestamp(definition, cursor, format):
	try:
		timestamp = definition.udate(cursor)
	except orasql.SQLObjectNotFoundError:
		return "doesn't exist"
	if timestamp is not None:
		timestamp = timestamp.strftime(format)
	else:
		timestamp = "without timestamp"
	return timestamp


def showudiff(definition, ddl1, ddl2, cursor1, cursor2, color, context=3, timeformat="%c"):
	ddl1 = ddl1.splitlines()
	ddl2 = ddl2.splitlines()

	filename1 = "%r in %s" % (definition, cursor1.connection.connectstring())
	filename2 = "%r in %s" % (definition, cursor2.connection.connectstring())

	timestamp1 = gettimestamp(definition, cursor1, timeformat)
	timestamp2 = gettimestamp(definition, cursor2, timeformat)

	for line in difflib.unified_diff(ddl1, ddl2, filename1, filename2, timestamp1, timestamp2, n=context, lineterm=""):
		if color:
			if line.startswith("---"):
				line = ansistyle.Text(color4removedfile, line)
			elif line.startswith("+++"):
				line = ansistyle.Text(color4addedfile, line)
			elif line.startswith("-"):
				line = ansistyle.Text(color4removedline, line)
			elif line.startswith("+"):
				line = ansistyle.Text(color4addedline, line)
			elif line.startswith("@@"):
				line = ansistyle.Text(color4pos, line)
		print line


if __name__ == "__main__":
	import sys, optparse

	colors = ("yes", "no", "auto")
	modes = ("brief", "udiff", "full")
	p = optparse.OptionParser(usage="usage: %prog [options] connectstring1 connectstring2")
	p.add_option("-v", "--verbose", dest="verbose", help="Give a progress report?", default=False, action="store_true")
	p.add_option("-c", "--color", dest="color", help="Color output (%s)" % ", ".join(colors), default="auto", choices=colors)
	p.add_option("-m", "--mode", dest="mode", help="Output mode (%s)" % ", ".join(modes), default="udiff", choices=modes)
	p.add_option("-n", "--context", dest="context", help="Number of context lines", type="int", default=2)

	(options, args) = p.parse_args()
	if len(args) != 2:
		p.error("incorrect number of arguments")
		sys.exit(1)

	if options.color == "yes":
		options.colorstdout = True
		options.colorstderr = True
	elif options.color == "no":
		options.colorstdout = False
		options.colorstderr = False
	else:
		try:
			import os
			options.colorstdout = os.isatty(sys.stdout.fileno())
			options.colorstderr = os.isatty(sys.stderr.fileno())
		except (KeyboardInterrupt, SystemExit):
			raise
		except Exception:
			options.colorstdout = False
			options.colorstderr = False

	connection1 = orasql.connect(args[0])
	cursor1 = connection1.cursor()

	connection2 = orasql.connect(args[1])
	cursor2 = connection2.cursor()

	def fetch(cursor):
		objects = set()

		for definition in cursor.iterobjects(mode="flat", all=False):
			if options.verbose:
				msg = "%s: %s: %r" % (sys.argv[0], cs(cursor, options.colorstderr), definition)
				if definition.owner is not None:
					msg += " (skipped)"
				print >>sys.stderr, msg
			if definition.owner is None:
				objects.add(definition)
		return objects

	objects1 = fetch(cursor1)
	objects2 = fetch(cursor2)

	for definition in objects1 - objects2:
		if options.mode == "brief":
			print "%r: only in %s" % (definition, cs(cursor1, options.colorstdout))
		elif options.mode == "full":
			print comment("%r: only in %s" % (definition, cs(cursor1, options.colorstdout)), options.colorstdout)
			ddl = definition.dropddl(cursor1)
			if ddl:
				print "%s;" % ddl
		elif options.mode == "udiff":
			ddl = definition.createddl(cursor1)
			showudiff(definition, ddl, "", cursor1, cursor2, options.colorstdout, options.context)

	for definition in objects2 - objects1:
		if options.mode == "brief":
			print "%r: only in %s" % (definition, cs(cursor2, options.colorstdout))
		elif options.mode == "full":
			print comment("%r: only in %s" % (definition, cs(cursor2, options.colorstdout)), options.colorstdout)
			ddl = definition.createddl(cursor2)
			if ddl:
				print definition.createddl(cursor2)
		elif options.mode == "udiff":
			ddl = definition.createddl(cursor2)
			showudiff(definition, "", ddl, cursor1, cursor2, options.colorstdout, options.context)

	for definition in objects1 & objects2:
		ddl1 = definition.createddl(cursor1)
		ddl2 = definition.createddl(cursor2)
		if ddl1 != ddl2:
			if options.mode == "brief":
				print "%r: different" % definition
			elif options.mode == "full":
				print comment("%r: different" % definition, options.colorstdout)
				print definition.createddl(cursor2)
			elif options.mode == "udiff":
				showudiff(definition, ddl1, ddl2, cursor1, cursor2, options.colorstdout, options.context)
